"""
SPADL - Sane Python adapter to DbgLog

Allows to log using standard Python logging to DbgLog
(http://dbglog.sourceforge.net/).

"""

import logging
import sys

from dbglog import dbg


DEBUG = 'DBG'
INFO = 'INFO'
WARNING = 'WARN'
ERROR = 'ERR'
FATAL = 'FATAL'

DBG_LOG_LEVELS = {
    logging.DEBUG: DEBUG,
    logging.INFO: INFO,
    logging.WARNING: WARNING,
    logging.ERROR: ERROR,
    logging.FATAL: FATAL,
}

if sys.version_info >= (3,):
    INTEGER_TYPES = int
else:
    INTEGER_TYPES = (int, long)

# Default format
# It contains only information which is not present in DbgLog output.
BASIC_FORMAT = '%(name)s: %(message)s'


def configure(severity=1, level=None, format=BASIC_FORMAT, datefmt=None):
    """
    Configure Python logging to write to DbgLog.

    This function adds a configured `DbgLogHandler` to the root logger.
    It can be used as a shortcut in common cases when detailed
    configuration is not necessary.

    Note that this method does not configure DbgLog itself.

    :param severity: int or dict, for the `DbgLogHandler` constructor
    :param level: sets level of the root logger if not None
    :param format: given to the handler's formatter
    :param datefmt: given to the handler's formatter

    """
    handler = DbgLogHandler(severity)
    formatter = logging.Formatter(format, datefmt)
    handler.setFormatter(formatter)
    root = logging.getLogger()
    root.addHandler(handler)
    if level is not None:
        root.setLevel(level)


class DbgLogHandler(logging.Handler):
    """
    A handler class which writes logging records to DbgLog.

    :param severity: A dictionary mapping logger names to severities
            (numbers 1-4 inclusive, use 0 for ignoring specific logger).
            Alternatively an integer number can be given and it will
            be used as default severity for all loggers.

    """

    def __init__(self, severity):
        logging.Handler.__init__(self)
        if isinstance(severity, INTEGER_TYPES):
            self.severities = {'': severity}
        else:
            self.severities = severity

    def emit(self, record):
        """
        Emits a record.

        Logs the record using the DbgLog library.

        """
        try:
            severity = self.getDbgLogSeverity(record)
            if not severity > 0:
                return
            level = self.getDbgLogLevel(record)
            if not level:
                return
            kwargs = {level: severity}
            # Do not format message if it wont be logged.
            if not dbg.checkLevel(**kwargs):
                return
            # The message is already formated, prevent formatting by DbgLog
            msg = self.format(record).replace('%', '%%')
            location = (record.filename, record.funcName, record.lineno)
            dbg.log(msg, location=location, **kwargs)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)

    def getDbgLogLevel(self, record):
        """
        Returns name of DbgLog level.

        Maps a standard logging level set to a `logging.LogRecord`
        instance to one of 'FATAL', 'ERR', 'WARN', 'INFO', 'DBG' or ''.

        """
        level = record.levelno
        try:
            return DBG_LOG_LEVELS[level // 10 * 10]
        except KeyError:
            return FATAL if level > logging.FATAL else ''

    def getDbgLogSeverity(self, record):
        """
        Returns severity for DbgLog.

        Maps a logger name set to a `logging.LogRecord` instance
        to a severity number from 1 to 4 (inclusive).

        """
        severities = self.severities
        name = record.name or 'root'
        try:
            return severities[name]
        except KeyError:
            pass
        prefix = name
        while prefix:
            try:
                prefix = prefix[:prefix.rindex('.')]
            except ValueError:
                prefix = ''
            try:
                severity = severities[prefix]
            except KeyError:
                continue
            else:
                severities[name] = severity
                return severity
        return 0




import logging

import spadl
from dbglog import dbg


# Configure Python logging
#
# The `spadl.configure` function can be used or the `spadl.DbgLogHandler`
# handler be created and registered using any method in the
# `logging.config` module.

spadl.configure({
    # Log records from the 'app' logger and its children will
    # be logged using the highest severity (D4, I4, W4, E4, F4).
    'app': 4,
    # But log records from the 'app.request' (and its children)
    # will be logged using a lower severity (D3, I3, W3, E3, F3).
    'app.request': 3,
    # Another severity for another loggers.
    'rpc': 2,
    # Log records from the 'db.sql' logger will be ignored.
    'db.sql': 0,
    # Specification of the default severity. If was not present here then
    # the default behaviour would be to ignore records from unknown loggers.
    '': 1,
}, level=logging.DEBUG)


# Configure DbgLog.
#
# Messages are written to error output for demonstration purposes
# but any DbgLog configuration is possible.
dbg.logStderr(True)
# It is a good idea to set a low DbgLog mask and filter records
# using standard logging levels. The effective logging levels can be
# easily checked using the `logging.Logger.isEnabledFor` method but
# testing DbgLog mask is insufficient if there are other logging handlers.
dbg.logMask('F1E1W1I1D1')


def demo():
    logging.debug('Hello world, this is logged as D1.')
    logging.getLogger('app.server').info('This application message is I4.')
    logging.getLogger('app.request').warning('But this message is W3.')
    logging.getLogger('rpc.stats').error('This is configured as E2.')
    logging.getLogger('db.connection').fatal('Level of this defaults to F1.')
    logging.getLogger('db.sql').debug('This is ignored and lost forever.')
    logging.log(25, 'Custom level is not a problem.')
    try:
        infinity = 1 / 0
    except ZeroDivisionError:
        logging.exception('Exceptions are logged too.')
    logging.getLogger('app').info('The last message must be %s!', 'GOOD')


if __name__ == '__main__':
    demo()



#!/usr/bin/env python

import sys
from setuptools import setup

VERSION = (0, 3)
VERSION_STR = ".".join(map(str, VERSION))


url = 'https://github.com/mila/spadl'

try:
    if sys.version_info >= (3,):
        long_description = open('README.rst', 'rb').read().decode('utf-8')
    else:
        long_description = open('README.rst', 'r').read().decode('utf-8')
except IOError:
    long_description = "See %s" % url


setup(
    name='spadl',
    version=VERSION_STR,
    description='This package provides a standard logging handler which writes log records to DbgLog.',
    long_description=long_description,
    author='Miloslav Pojman',
    author_email='miloslav.pojman@gmail.com',
    url=url,
    py_modules=['spadl'],
    include_package_data=True,
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: System :: Logging'
    ],
    zip_safe=False,
)




import re
import os
import errno
import logging
import tempfile
import unittest

from dbglog import dbg

import spadl


class DbgLogHandlerTestCase(unittest.TestCase):
    """
    Tests for the the `DbgLogHandler` class.
    """

    def setUp(self):
        # Configure Python logging
        logger = logging.getLogger()
        self.origLevel, logger.level = logger.level, logging.NOTSET
        self.handlers = []
        # Configure DbgLog
        self.origLogMask = dbg.getLogMask()
        dbg.logMask('F1E1W1I1D1')
        self.origLogFile = dbg.getLogFile()
        self.filename = tempfile.mktemp(suffix='.log')
        dbg.logFile(self.filename)

    def tearDown(self):
        # Restore Python logging configuration
        logger = logging.getLogger()
        logger.level = self.origLevel
        for handler in self.handlers:
            logger.removeHandler(handler)
        # Restore DbgLog configuration
        dbg.logMask(self.origLogMask)
        dbg.logFile(self.origLogFile or '')
        try:
            os.remove(self.filename)
        except EnvironmentError:
            pass

    def addHandler(self, severity=1):
        handler = spadl.DbgLogHandler(severity)
        logging.getLogger().addHandler(handler)
        self.handlers.append(handler)

    def readLogFile(self):
        try:
            with open(self.filename, 'r') as f:
                return f.read()
        except EnvironmentError as e:
            if e.errno == errno.ENOENT:
                return ''
            raise

    def getMessagePattern(self, message, level=None):
        if level is None:
            return '%s.* {%s' % (message, __file__)
        return '%s:.* %s.* {%s' % (level, message, __file__)

    def assertLogMessage(self, message, level=None):
        content = self.readLogFile()
        pattern = self.getMessagePattern(message, level)
        self.assertTrue(re.search(pattern, content),
                        'Message %r was not found in log.' % message)

    def assertNoLogMessage(self, message, level=None):
        content = self.readLogFile()
        pattern = self.getMessagePattern(message, level)
        self.assertFalse(re.search(pattern, content),
                         'Message %r was found in log.' % message)

    def testDebugIsLogged(self):
        self.addHandler()
        logging.debug('Example debug message')
        self.assertLogMessage('Example debug message', 'D1')

    def testInfoIsLogged(self):
        self.addHandler()
        logging.info('Example info message')
        self.assertLogMessage('Example info message', 'I1')

    def testWarningIsLogged(self):
        self.addHandler()
        logging.warning('Example warning message')
        self.assertLogMessage('Example warning message', 'W1')

    def testErrorIsLogged(self):
        self.addHandler()
        logging.error('Example error message')
        self.assertLogMessage('Example error message', 'E1')

    def testFatalIsLogged(self):
        self.addHandler()
        logging.fatal('Example fatal message')
        self.assertLogMessage('Example fatal message', 'F1')

    def testNegativeLevelIsNotLogged(self):
        self.addHandler()
        logging.log(-1, 'Example message')
        self.assertNoLogMessage('Example message')

    def testZeroLevelIsNotLogged(self):
        self.addHandler()
        logging.log(0, 'Example message')
        self.assertNoLogMessage('Example message')

    def testLevelUnderDebugNotIsLogged(self):
        self.addHandler()
        logging.log(logging.DEBUG - 1, 'Example message')
        self.assertNoLogMessage('Example message')

    def testLevelOverDebugIsLogged(self):
        self.addHandler()
        logging.log(logging.DEBUG + 1, 'Example message')
        self.assertLogMessage('Example message', 'D1')

    def testLevelUnderInfoIsLogged(self):
        self.addHandler()
        logging.log(logging.INFO - 1, 'Example message')
        self.assertLogMessage('Example message', 'D1')

    def testLevelOverInfoIsLogged(self):
        self.addHandler()
        logging.log(logging.INFO + 1, 'Example message')
        self.assertLogMessage('Example message', 'I1')

    def testLevelUnderWarningIsLogged(self):
        self.addHandler()
        logging.log(logging.WARNING - 1, 'Example message')
        self.assertLogMessage('Example message', 'I1')

    def testLevelOverWarningIsLogged(self):
        self.addHandler()
        logging.log(logging.WARNING + 1, 'Example message')
        self.assertLogMessage('Example message', 'W1')

    def testLevelUnderErrorIsLogged(self):
        self.addHandler()
        logging.log(logging.ERROR - 1, 'Example message')
        self.assertLogMessage('Example message', 'W1')

    def testLevelOverErrorIsLogged(self):
        self.addHandler()
        logging.log(logging.ERROR + 1, 'Example message')
        self.assertLogMessage('Example message', 'E1')

    def testLevelUnderFatalIsLogged(self):
        self.addHandler()
        logging.log(logging.FATAL - 1, 'Example message')
        self.assertLogMessage('Example message', 'E1')

    def testLevelOverFatalIsLogged(self):
        self.addHandler()
        logging.log(logging.FATAL + 1, 'Example message')
        self.assertLogMessage('Example message', 'F1')

    def testHugeLevelIsLogged(self):
        self.addHandler()
        logging.log(100, 'Example message')
        self.assertLogMessage('Example message', 'F1')

    def testRootLoggerWithZeroSeverity(self):
        self.addHandler(0)
        logging.debug('Example message')
        self.assertNoLogMessage('Example message')

    def testRootLoggerWithSeverityEqualToOne(self):
        self.addHandler(1)
        logging.debug('Example message')
        self.assertLogMessage('Example message', 'D1')

    def testRootLoggerWithSeverityEqualToTwo(self):
        self.addHandler(2)
        logging.info('Example message')
        self.assertLogMessage('Example message', 'I2')

    def testRootLoggerWithSeverityEqualToThree(self):
        self.addHandler(3)
        logging.warning('Example message')
        self.assertLogMessage('Example message', 'W3')

    def testRootLoggerWithSeverityEqualToFour(self):
        self.addHandler(4)
        logging.error('Example message')
        self.assertLogMessage('Example message', 'E4')

    def testCustomLoggerWithZeroSeverity(self):
        self.addHandler(0)
        logging.getLogger('foo.bar').debug('Example message')
        self.assertNoLogMessage('Example message')

    def testCustomLoggerWithSeverityEqualToOne(self):
        self.addHandler(1)
        logging.getLogger('foo.bar').debug('Example message')
        self.assertLogMessage('Example message', 'D1')

    def testCustomLoggerWithSeverityEqualToTwo(self):
        self.addHandler(2)
        logging.getLogger('foo.bar').info('Example message')
        self.assertLogMessage('Example message', 'I2')

    def testCustomLoggerWithSeverityEqualToThree(self):
        self.addHandler(3)
        logging.getLogger('foo.bar').warning('Example message')
        self.assertLogMessage('Example message', 'W3')

    def testCustomLoggerWithSeverityEqualToFour(self):
        self.addHandler(4)
        logging.getLogger('foo.bar').error('Example message')
        self.assertLogMessage('Example message', 'E4')

    def testRootLoggerWithoutRootSeverity(self):
        self.addHandler({})
        logging.info('Example message')
        self.assertNoLogMessage('Example message')

    def testRootLoggerWithRootSeverity(self):
        self.addHandler({'': 4})
        logging.info('Example message')
        self.assertLogMessage('Example message', 'I4')

    def testCustomLoggerWithoutRootSeverity(self):
        self.addHandler({})
        logging.getLogger('foo.bar').info('Example message')
        self.assertNoLogMessage('Example message')

    def testCustomLoggerWithRootSeverity(self):
        self.addHandler({'': 4})
        logging.getLogger('foo.bar').info('Example message')
        self.assertLogMessage('Example message', 'I4')

    def testSeverityFromGivenLoggerIsUsed(self):
        self.addHandler({'foo.bar': 4, 'foo': 3, '': 2})
        logging.getLogger('foo.bar').info('Example message')
        self.assertLogMessage('Example message', 'I4')

    def testSeverityFromParentLoggerIsUsed(self):
        self.addHandler({'foo.bar': 4, 'foo': 3, '': 2})
        logging.getLogger('foo.baz').info('Example message')
        self.assertLogMessage('Example message', 'I3')

    def testSeverityFromRootLoggerIsUsed(self):
        self.addHandler({'foo.bar': 4, 'foo': 3, '': 2})
        logging.getLogger('baz.bar').info('Example message')
        self.assertLogMessage('Example message', 'I2')

    def testSeverityFromSimilarLoggerIsNotUsed(self):
        self.addHandler({'foo.bar': 4, 'foo': 3, '': 2})
        logging.getLogger('foooo').info('Example message')
        self.assertLogMessage('Example message', 'I2')

    def testMessageWithParams(self):
        self.addHandler()
        logging.info('"%s %s"', 'hello', 'world')
        self.assertLogMessage('"hello world"')

    def testMessageWithNamedParams(self):
        self.addHandler()
        logging.info('"%(h)s %(w)s"', {'h':'hello', 'w': 'world'})
        self.assertLogMessage('"hello world"')

    def testMessageWithPercentSign(self):
        self.addHandler()
        logging.info('"100%"')
        self.assertLogMessage('"100%"')

    def testMessageWithPercentSignAndParams(self):
        self.addHandler()
        logging.info('"100%% %s %s"', 'hello', 'world')
        self.assertLogMessage('"100% hello world"')

    def testMessageWithPercentSignAndNamedParams(self):
        self.addHandler()
        logging.info('"100%% %(h)s %(w)s"', {'h':'hello', 'w': 'world'})
        self.assertLogMessage('"100% hello world"')


if __name__ == '__main__':
    unittest.main()
